@@ -145,6 +145,10 @@ let output o s p l =
 	if p + l > sl || p < 0 || l < 0 then invalid_arg "IO.output";
 	o.out_output s p l
 
+let scanf i fmt =
+	let ib = Scanf.Scanning.from_function (fun () -> try read i with No_more_input -> raise End_of_file) in
+	Scanf.kscanf ib (fun _ exn -> raise exn) fmt
+
 let printf o fmt =
 	Printf.kprintf (fun s -> nwrite o s) fmt
 
@@ -248,6 +252,33 @@ let output_string() =
 		out_flush = (fun () -> ());
 	}
 
+let output_strings() =
+	let sl = ref [] in
+	let size = ref 0 in
+	let b = Buffer.create 0 in
+	{
+		out_write = (fun c ->
+			if !size = Sys.max_string_length then begin
+				sl := Buffer.contents b :: !sl;
+				Buffer.clear b;
+				size := 0;
+			end else incr size;
+			Buffer.add_char b c
+		);
+		out_output = (fun s p l ->
+			if !size + l > Sys.max_string_length then begin
+				sl := Buffer.contents b :: !sl;
+				Buffer.clear b;
+				size := 0;
+			end else size := !size + l;
+			Buffer.add_substring b s p l;
+			l
+		);
+		out_close = (fun () -> sl := Buffer.contents b :: !sl; List.rev (!sl));
+		out_flush = (fun () -> ());
+	}
+
+
 let input_channel ch =
 	{
 		in_read = (fun () ->
@@ -464,6 +495,9 @@ let read_i64 ch =
 	let big = Int64.of_int32 (read_real_i32 ch) in
 	Int64.logor (Int64.shift_left big 32) small
 
+let read_float32 ch =
+	Int32.float_of_bits (read_real_i32 ch)
+
 let read_double ch =
 	Int64.float_of_bits (read_i64 ch)
 
@@ -509,6 +543,9 @@ let write_i64 ch n =
 	write_real_i32 ch (Int64.to_int32 n);
 	write_real_i32 ch (Int64.to_int32 (Int64.shift_right_logical n 32))
 
+let write_float32 ch f =
+	write_real_i32 ch (Int32.bits_of_float f)
+
 let write_double ch f =
 	write_i64 ch (Int64.bits_of_float f)
 
@@ -562,6 +599,9 @@ let read_i64 ch =
 	let small = Int64.logor base (Int64.shift_left (Int64.of_int ch4) 24) in
 	Int64.logor (Int64.shift_left big 32) small
 
+let read_float32 ch =
+	Int32.float_of_bits (read_real_i32 ch)
+
 let read_double ch =
 	Int64.float_of_bits (read_i64 ch)
 
@@ -595,6 +635,9 @@ let write_i64 ch n =
 	write_real_i32 ch (Int64.to_int32 (Int64.shift_right_logical n 32));
 	write_real_i32 ch (Int64.to_int32 n)
 
+let write_float32 ch f =
+	write_real_i32 ch (Int32.bits_of_float f)
+
 let write_double ch f =
 	write_i64 ch (Int64.bits_of_float f)
 
