@@ -53,7 +53,7 @@ let ends_with s e =
     in
     loop s e 0
 
-let find str sub =
+let find_from str pos sub =
 	let sublen = length sub in
 	if sublen = 0 then
 		0
@@ -61,7 +61,7 @@ let find str sub =
 		let found = ref 0 in
 		let len = length str in
 		try
-			for i = 0 to len - sublen do
+			for i = pos to len - sublen do
 				let j = ref 0 in
 				while unsafe_get str (i + !j) = unsafe_get sub !j do
 					incr j;
@@ -72,6 +72,8 @@ let find str sub =
 		with
 			Exit -> !found
 
+let find str sub = find_from str 0 sub
+
 let exists str sub =
 	try
 		ignore(find str sub);
@@ -99,18 +101,17 @@ let split str sep =
 	sub str 0 p, sub str (p + len) (slen - p - len)
 
 let nsplit str sep =
-	if str = "" then []
-	else if sep = "" then raise Invalid_string
-	else (
-		let rec nsplit str sep =
-			try
-				let s1 , s2 = split str sep in
-				s1 :: nsplit s2 sep
-			with
-				Invalid_string -> [str]
+  if str = "" then []
+  else if sep = "" then raise Invalid_string
+  else
+    let rec loop acc pos =
+      if pos > String.length str then
+        List.rev acc
+      else
+        let i = try find_from str pos sep with Invalid_string -> String.length str in
+        loop (String.sub str pos (i - pos) :: acc) (i + String.length sep)
 		in
-		nsplit str sep
-	)
+    loop [] 0
 
 let join = concat
 
